#!/usr/bin/env python

import os
import sys
import ast
import time

##########################################################################
# Python script to parse the Sonar metrics files.
#
# 2 input parameters required:
#    - Directory name of the metrics files
#    - Directory name of the output location.
#
# Output generates 3 files:
#    - 1 ascii file (sonar_sca_metrics.txt)
#    - 1 comma-separated file (sonar_sca_metrics.csv)
#    - 1 semicolon-separated file (sonar_sca_metrics_semicolon.csv)
#
# The last two files can directly be opened in Excel.  Depending on the list
# separator settings for your country/location, you should open one or the other.
#
# Geert Vancompernolle, September 2013
##########################################################################

class Usage:
    def __init__(self):
        print "Missing parameter!!!"
        print ''
        print 'Usage:'
        print '    %s <metricsdir> <outputdir>' % sys.argv[0]
        print ''
        print 'Where:'
        print '    <directoryname> is the directory containing the Sonar metrics files.'
        print '    <outputdir> is the directory where the Sonar SCA result files will be written.'
        print ''
        print 'Pls. restart the app with the correct parameter...'


class Log:
    """
    Class Log implements a write() method that immedately flushes
    the content to a file to ensure that everything is logged,
    even during an unexpected exit.
    The close() method is added for completeness.
    """
    def __init__( self, f, g, h, a = 'r' ):
        self.txtfile = f
        self.csvfile = g
        self.ssvfile = h
        try:
            os.stat(f)
            os.remove( f )
        except OSError:
            pass
        try:
            os.stat(g)
            os.remove( g )
        except OSError:
            pass
        try:
            os.stat(h)
            os.remove( h )
        except OSError:
            pass

    def write(self, lst):
        try:
            # Open the file in append mode
            self.txtf = open(self.txtfile, 'a')
            if len(lst) == 5:
                self.txtf.write('{:25s} {:25s} {:25s} {:25s} {:25s}'.format(*lst) + '\n')
            elif len(lst) == 2:
                self.txtf.write('{:25s} {:25s}'.format(*lst) + '\n')
            else:
                self.txtf.write(lst[0] + '\n')
            self.txtf.close()

            self.csvf = open(self.csvfile, 'a')
            self.csvf.write(','.join(lst) + '\n')
            self.csvf.close()

            self.ssvf = open(self.ssvfile, 'a')
            self.ssvf.write(';'.join(lst) + '\n')
            self.ssvf.close()
        except IOError,  (errno,  strerror):
            print 'Not possible to open log file "%s".  I/O error (%s): "%s"' % (self.file,  errno,  strerror)


class SonarMetrics():
    def __init__(self, metricsdir, outputdir):
        self.MetricsDir = metricsdir
        self.OutputDir = outputdir
        self.MetricsFiles = None
        self.SonarMetricsFiles = Log(self.OutputDir + os.sep + 'sonar_sca_metrics.txt'
                                    ,self.OutputDir + os.sep + 'sonar_sca_metrics.csv'
                                    ,self.OutputDir + os.sep + 'sonar_sca_metrics_semicolon.csv'
                                    ,'w'
                                    )
        self.SonarMetricsFiles.write(['*********************************************   S O N A R   M E T R I C S   *********************************************'])
        self.SonarMetricsFiles.write([''])
        self.SonarMetricsFiles.write(['Static code analysis covers: Java (not yet C/C++)'])
        self.SonarMetricsFiles.write(['Generated from : %s' % self.MetricsDir])
        self.SonarMetricsFiles.write(['Generation time: %s' % time.strftime('%A %d %b %Y %H:%M:%S')])
        self.SonarMetricsFiles.write([''])
        self.SonarMetricsFiles.write(['PACKAGE (*)','ANALYSIS','BLOCKING','CRITICAL','MAJOR'])
        self.SonarMetricsFiles.write(['===========','========','========','========','====='])

    def errorMessage( self, filecontent ):
        if filecontent:
            splitresult = filecontent.split(None,1)
            if "Wget" in splitresult[1]:
                self.SonarMetricsFiles.write([splitresult[0], 'Missing (Sonar db access failed)'])
            else:
                self.SonarMetricsFiles.write([splitresult[0], 'Missing (no properties file found)'])
        else:
            self.SonarMetricsFiles.write(['Could not open file containing the metrics information'])

    def collectFiles(self):
        print " ==> Collecting the files to be processed from directory \"{}\"".format(self.MetricsDir)
        try:
            self.MetricsFiles = os.listdir(self.MetricsDir)
        except EnvironmentError:
            print 'Failed to get file listing'
            exit(1)

    def processFiles(self):
        print " ==> Processing the input files"
        try:
            totalBlocking = 0
            totalCritical = 0
            totalMajor = 0
            for item in self.MetricsFiles:
                with open( self.MetricsDir + os.sep + item, 'r' ) as f:
                    filecontent = f.read()
                    try:
                        result = ast.literal_eval(filecontent)
                    except:
                        print 'Exception trapped: {}'.format(filecontent)
                        self.errorMessage(filecontent)
                    else:
                        if isinstance( result, list ):
                            if 'msr' in result[0]:
                                msr = result[0]['msr']
                                pkgname = result[0]['name']
                                issues = []
                                if len( msr ) > 0:
                                    for x in msr:
                                        issueValue = x['frmt_val'].replace(',','')
                                        if x['key'] == 'blocker_violations':
                                            totalBlocking += int(issueValue)
                                        if x['key'] == 'critical_violations':
                                            totalCritical += int(issueValue)
                                        if x['key'] == 'major_violations':
                                            totalMajor += int(issueValue)
                                        issues.append(issueValue)
#                                issues = [i.replace(',','') for i in issues]
                                self.SonarMetricsFiles.write([pkgname, 'Done =>', issues[0], issues[1], issues[2]])
                        else:
                            self.errorMessage(filecontent)
            self.SonarMetricsFiles.write(['===========','========','========','========','====='])
            self.SonarMetricsFiles.write(['TOTAL','',str(totalBlocking),str(totalCritical),str(totalMajor)])
        except EnvironmentError:
            self.errorMessage(None)

    def finishProcessing(self):
        print ' ==> Finalising the processing'
        self.SonarMetricsFiles.write([''])
        self.SonarMetricsFiles.write(['(*) Package name is derived from the property \"sonar.projectName\" from the properties file.'])
        self.SonarMetricsFiles.write(['    If no properties file is present => last part of the package directory name is used.'])
        print ' ==> Finished'
        print ''
        print ' Resulting files:'
        print ' ----------------'
        print '   sonar_sca_metrics.txt'
        print '   sonar_sca_metrics.csv            (comma separated values)'
        print '   sonar_sca_metrics_semicolon.csv  (semicolon separated values)'
        print ''




if __name__ == "__main__":
    print('\n')
    print('**********************************')
    print('Sonar SCA metrics report generator')
    print('**********************************')
    print('\n')
    if len(sys.argv) != 3:
        Usage()
        exit(1)
    else:
        if not os.path.isdir(sys.argv[1]):
            print "Input parameter is not a directory!"
            exit(1)
        else:
            metrics = SonarMetrics(sys.argv[1], sys.argv[2])
            metrics.collectFiles()
            metrics.processFiles()
            metrics.finishProcessing()
